@@ -2,9 +2,14 @@
 Parent classes for all families.
 """
 # pylint: disable=no-member
+# pylint: disable=import-outside-toplevel
 from collections import namedtuple
+from copy import copy
 
-from ipywidgets import interactive
+try:
+    from ipywidgets import interactive
+except ImportError:
+    pass
 import numpy as np
 
 from ..internal.plot_helper import (
@@ -18,9 +23,10 @@ from ..internal.distribution_helper import (
     init_vals,
     valid_scalar_params,
     valid_distribution,
-    hdi_from_pdf,
 )
 
+from ..internal.optimization import optimize_hdi
+
 
 class Distribution:
     """
@@ -30,54 +36,80 @@ class Distribution:
     """
 
     def __init__(self):
-        self.rv_frozen = None
         self.is_frozen = False
         self.opt = None
 
     def __repr__(self):
         name = self.__class__.__name__
+        if name in ["Truncated", "Censored", "Hurdle"]:
+            name += self.dist.__class__.__name__
         if self.is_frozen:
             bolded_name = "\033[1m" + name + "\033[0m"
 
             description = "".join(
-                f"{n}={np.round(v, 2)}," for n, v in zip(self.param_names, self.params)
-            ).strip(",")
+                f"{n}={v:.3g}, "
+                if np.isscalar(v)
+                else f"{n}=[" + "".join(f"{vi:.3g}, " for vi in v).strip(", ") + "], "
+                for n, v in zip(self.param_names, self.params)
+            ).strip(", ")
 
             return f"{bolded_name}({description})"
         else:
             return name
 
-    def _update_rv_frozen(self):
-        """Update the rv_frozen object"""
+    @property
+    def params_dict(self):
+        if self.is_frozen:
+            return dict(zip(self.param_names, self.params))
+        else:
+            return None
 
-        frozen = self._get_frozen()
-        if frozen is not None:
-            self.is_frozen = True
-            self.rv_frozen = frozen
-
-    def summary(self, fmt=".2f", mass=0.94):
+    def summary(self, mass=0.94, interval="hdi", fmt=".2f"):
         """
         Namedtuple with the mean, median, standard deviation, and lower and upper bounds
         of the equal-tailed interval.
 
         Parameters
         ----------
+        mass: float
+            Probability mass for the equal-tailed interval. Defaults to 0.94
+        interval : str or list-like
+            Type of interval. Available options are highest density interval `"hdi"` (default),
+            equal tailed interval `"eti"` or arbitrary interval defined by a list-like object
+            with a pair of values.
         fmt : str
             fmt used to represent results using f-string fmt for floats. Default to ".2f"
             i.e. 2 digits after the decimal point.
-        mass: float
-            Probability mass for the equal-tailed interval. Defaults to 0.94
         """
         valid_distribution(self)
 
+        if not isinstance(fmt, str):
+            raise ValueError("Invalid format string.")
+
         if valid_scalar_params(self):
-            attr = namedtuple(self.__class__.__name__, ["mean", "median", "std", "lower", "upper"])
-            mean = float(f"{self.rv_frozen.mean():{fmt}}")
-            median = float(f"{self.rv_frozen.median():{fmt}}")
-            std = float(f"{self.rv_frozen.std():{fmt}}")
-            eti = self.rv_frozen.interval(mass)
-            lower_tail = float(f"{eti[0]:{fmt}}")
-            upper_tail = float(f"{eti[1]:{fmt}}")
+            name = self.__class__.__name__
+            if name == "Truncated":
+                name = "Truncated" + self.dist.__class__.__name__
+            elif name == "Censored":
+                name = "Censored" + self.dist.__class__.__name__
+            attr = namedtuple(name, ["mean", "median", "std", "lower", "upper"])
+            mean = float(f"{self.mean():{fmt}}")
+            median = float(f"{self.median():{fmt}}")
+            std = float(f"{self.std():{fmt}}")
+
+            if isinstance(interval, (tuple, list, np.ndarray)):
+                c_int = self.ppf(interval)
+            elif interval == "hdi":
+                c_int = self.hdi(mass, fmt=fmt)
+            elif interval == "eti":
+                c_int = self.eti(mass, fmt=fmt)
+
+            if self.kind == "discrete":
+                lower_tail = c_int[0]
+                upper_tail = c_int[1]
+            else:
+                lower_tail = float(f"{c_int[0]:{fmt}}")
+                upper_tail = float(f"{c_int[1]:{fmt}}")
             return attr(mean, median, std, lower_tail, upper_tail)
         else:
             return None
@@ -92,7 +124,7 @@ class Distribution:
         random_state : {None, int, numpy.random.Generator, numpy.random.RandomState}
             Defaults to None
         """
-        return self.rv_frozen.rvs(*args, **kwds)
+        return self.rvs(*args, **kwds)
 
     def cdf(self, x, *args, **kwds):
         """Cumulative distribution function.
@@ -102,7 +134,7 @@ class Distribution:
         x : array_like
             Values on which to evaluate the cdf
         """
-        return self.rv_frozen.cdf(x, *args, **kwds)
+        return self.cdf(x, *args, **kwds)
 
     def ppf(self, q, *args, **kwds):
         """Percent point function (inverse of cdf).
@@ -112,50 +144,150 @@ class Distribution:
         x : array_like
             Values on which to evaluate the inverse of the cdf
         """
-        return self.rv_frozen.ppf(q, *args, **kwds)
+        return self.ppf(q, *args, **kwds)
 
-    def eti(self, fmt=".2f", mass=0.94):
+    def mean(self):
+        """Mean of the distribution."""
+        return self.mean()
+
+    def median(self):
+        """Median of the distribution."""
+        return self.median()
+
+    def std(self):
+        """Standard deviation of the distribution."""
+        return self.std()
+
+    def var(self):
+        """Variance of the distribution."""
+        return self.var()
+
+    def skewness(self):
+        """Skewness of the distribution."""
+        return self.stats(moment="s")
+
+    def kurtosis(self):
+        """Kurtosis of the distribution"""
+        return self.stats(moments="k")
+
+    def moments(self, types="mvsk"):
+        """
+        Compute moments of the distribution.
+
+        It can also return the standard deviation
+
+        Parameters
+        ----------
+        types : str
+            The type of moments to compute. Default is 'mvsk'
+            where 'm' = mean, 'v' = variance, 's' = skewness, and 'k' = kurtosis.
+            Valid combinations are any subset of 'mvsk'.
+        """
+        moments = []
+        for m_t in types:
+            if m_t not in "mdvsk":
+                raise ValueError(
+                    "The input string should only contain the letters "
+                    "'m', 'd', 'v', 's', or 'k'."
+                )
+            if m_t == "m":
+                moments.append(self.mean())
+            elif m_t == "d":
+                moments.append(self.std())
+            elif m_t == "v":
+                moments.append(self.var())
+            elif m_t == "s":
+                moments.append(self.skewness())
+            elif m_t == "k":
+                moments.append(self.kurtosis())
+
+        return moments
+
+    def eti(self, mass=0.94, fmt=".2f"):
         """Equal-tailed interval containing `mass`.
 
         Parameters
         ----------
-        fmt : str
-            fmt used to represent results using f-string fmt for floats. Default to ".2f"
-            i.e. 2 digits after the decimal point.
         mass: float
             Probability mass in the interval. Defaults to 0.94
+        fmt : str
+            fmt used to represent results using f-string fmt for floats. Default to ".2f"
+            i.e. 2 digits after the decimal point. Use `"none"` for no format.
         """
         valid_distribution(self)
 
+        if not isinstance(fmt, str):
+            raise ValueError("Invalid format string.")
+
         if valid_scalar_params(self):
-            eti = self.rv_frozen.interval(mass)
-            lower_tail = float(f"{eti[0]:{fmt}}")
-            upper_tail = float(f"{eti[1]:{fmt}}")
+            lower_tail, upper_tail = self.ppf([(1 - mass) / 2, 1 - (1 - mass) / 2])
+            if self.kind == "continuos" and fmt != "none":
+                print("hi!")
+                lower_tail = float(f"{lower_tail:{fmt}}")
+                upper_tail = float(f"{upper_tail:{fmt}}")
+            elif self.kind == "discrete":
+                lower_tail = int(lower_tail)
+                upper_tail = int(upper_tail)
+
             return (lower_tail, upper_tail)
         else:
             return None
 
-    def hdi(self, fmt=".2f", mass=0.94):
+    def hdi(self, mass=0.94, fmt=".2f"):
         """Highest density interval containing `mass`.
 
         Parameters
         ----------
-        fmt : str
-            fmt used to represent results using f-string fmt for floats. Default to ".2f"
-            i.e. 2 digits after the decimal point.
         mass: float
             Probability mass in the interval. Defaults to 0.94
+        fmt : str
+            fmt used to represent results using f-string fmt for floats. Default to ".2f"
+            i.e. 2 digits after the decimal point. Use `"none"` for no format.
         """
         valid_distribution(self)
 
+        if not isinstance(fmt, str):
+            raise ValueError("Invalid format string.")
+
         if valid_scalar_params(self):
-            hdi = hdi_from_pdf(self, mass)
-            lower_tail = float(f"{hdi[0]:{fmt}}")
-            upper_tail = float(f"{hdi[1]:{fmt}}")
+            lower_tail, upper_tail = optimize_hdi(self, mass)
+            if self.kind == "continuos" and fmt != "none":
+                lower_tail = float(f"{lower_tail:{fmt}}")
+                upper_tail = float(f"{upper_tail:{fmt}}")
             return (lower_tail, upper_tail)
         else:
             return None
 
+    def to_pymc(self, name=None, **kwargs):
+        """
+        Convert the PreliZ distribution to a PyMC distribution.
+
+        name : str
+            Name of PyMC distribution. Needed if inside Model context
+        kwargs : PyMC distributions properties
+            kwargs are used to specify properties such as shape or dims
+
+        Returns
+        -------
+        PyMC distribution
+        """
+        try:
+            import pymc.distributions as pm_dists
+            from pymc.model import Model
+
+            model = Model.get_context(error_if_none=False)
+
+            if model is None:
+                return getattr(pm_dists, self.__class__.__name__).dist(**self.params_dict, **kwargs)
+            else:
+                return getattr(pm_dists, self.__class__.__name__)(
+                    name, **self.params_dict, **kwargs
+                )
+        except ImportError:
+            pass
+
+        return None
+
     def _check_endpoints(self, lower, upper, raise_error=True):
         """
         Evaluate if the lower and upper values are in the support of the distribution
@@ -223,8 +355,10 @@ class Distribution:
         pointinterval=False,
         interval="hdi",
         levels=None,
-        support="full",
+        support="restricted",
         legend="legend",
+        color=None,
+        alpha=1,
         figsize=None,
         ax=None,
     ):
@@ -242,7 +376,7 @@ class Distribution:
             plot the median and two interquantiles ranges.
         interval : str
             Type of interval. Available options are highest density interval `"hdi"` (default),
-        equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+            equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
@@ -253,6 +387,11 @@ class Distribution:
         legend : str
             Whether to include a string with the distribution and its parameter as a ``"legend"`` a
             ``"title"`` or not include them ``None``.
+        color : str
+            Valid matplotlib color. Defaults to None, colors will be selected from the active
+            color cycle.
+        alpha : float
+            Transparency of the line. Defaults to 1 (no transparency).
         figsize : tuple
             Size of the figure
         ax : matplotlib axes
@@ -260,7 +399,17 @@ class Distribution:
 
         if valid_scalar_params(self):
             return plot_pdfpmf(
-                self, moments, pointinterval, interval, levels, support, legend, figsize, ax
+                self,
+                moments,
+                pointinterval,
+                interval,
+                levels,
+                support,
+                legend,
+                color,
+                alpha,
+                figsize,
+                ax,
             )
         else:
             return None
@@ -271,8 +420,10 @@ class Distribution:
         pointinterval=False,
         interval="hdi",
         levels=None,
-        support="full",
+        support="restricted",
         legend="legend",
+        color=None,
+        alpha=1,
         figsize=None,
         ax=None,
     ):
@@ -290,7 +441,7 @@ class Distribution:
             plot the median and two interquantiles ranges.
         interval : str
             Type of interval. Available options are highest density interval `"hdi"` (default),
-        equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+            equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
@@ -298,6 +449,11 @@ class Distribution:
         support : str:
             If ``full`` use the finite end-points to set the limits of the plot. For unbounded
             end-points or if ``restricted`` use the 0.001 and 0.999 quantiles to set the limits.
+        color : str
+            Valid matplotlib color. Defaults to None, colors will be selected from the active
+            color cycle.
+        alpha : float
+            Transparency of the line. Defaults to 1 (no transparency).
         legend : str
             Whether to include a string with the distribution and its parameter as a ``"legend"`` a
             ``"title"`` or not include them ``None``.
@@ -307,7 +463,17 @@ class Distribution:
         """
         if valid_scalar_params(self):
             return plot_cdf(
-                self, moments, pointinterval, interval, levels, support, legend, figsize, ax
+                self,
+                moments,
+                pointinterval,
+                interval,
+                levels,
+                support,
+                legend,
+                color,
+                alpha,
+                figsize,
+                ax,
             )
         else:
             return None
@@ -319,6 +485,8 @@ class Distribution:
         interval="hdi",
         levels=None,
         legend="legend",
+        color=None,
+        alpha=1,
         figsize=None,
         ax=None,
     ):
@@ -336,7 +504,7 @@ class Distribution:
             plot the median and two interquantiles ranges.
         interval : str
             Type of interval. Available options are highest density interval `"hdi"` (default),
-        equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+            equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
@@ -344,17 +512,30 @@ class Distribution:
         legend : str
             Whether to include a string with the distribution and its parameter as a ``"legend"`` a
             ``"title"`` or not include them ``None``.
+        color : str
+            Valid matplotlib color. Defaults to None, colors will be selected from the active
+            color cycle.
+        alpha : float
+            Transparency of the line. Defaults to 1 (no transparency).
         figsize : tuple
             Size of the figure
         ax : matplotlib axes
         """
         if valid_scalar_params(self):
-            return plot_ppf(self, moments, pointinterval, interval, levels, legend, figsize, ax)
+            return plot_ppf(
+                self, moments, pointinterval, interval, levels, legend, color, alpha, figsize, ax
+            )
         else:
             return None
 
     def plot_interactive(
-        self, kind="pdf", fixed_lim="both", pointinterval=True, interval="hdi", levels=None
+        self,
+        kind="pdf",
+        xy_lim="both",
+        pointinterval=True,
+        interval="hdi",
+        levels=None,
+        figsize=None,
     ):
         """
         Interactive exploration of distributions parameters
@@ -363,33 +544,40 @@ class Distribution:
         ----------
         kind : str:
             Type of plot. Available options are `pdf`, `cdf` and `ppf`.
-        fixed_lim : str or tuple
+        xy_lim : str or tuple
             Set the limits of the x-axis and/or y-axis.
             Defaults to `"both"`, the limits of both axis are fixed.
             Use `"auto"` for automatic rescaling of x-axis and y-axis.
             Or set them manually by passing a tuple of 4 elements,
-            the first two fox x-axis, the last two for x-axis. The tuple can have `None`.
+            the first two for x-axis, the last two for y-axis. The tuple can have `None`.
         pointinterval : bool
             Whether to include a plot of the quantiles. Defaults to False. If True the default is to
             plot the median and two interquantiles ranges.
         interval : str
             Type of interval. Available options are highest density interval `"hdi"` (default),
-        equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
+            equal tailed interval `"eti"` or intervals defined by arbitrary `"quantiles"`.
         levels : list
             Mass of the intervals. For hdi or eti the number of elements should be 2 or 1.
             For quantiles the number of elements should be 5, 3, 1 or 0
             (in this last case nothing will be plotted).
+        figsize : tuple
+            Size of the figure
         """
         check_inside_notebook()
 
         if valid_scalar_params(self, check_frozen=False):
             args = dict(zip(self.param_names, self.params))
         else:
-            args = init_vals[self.__class__.__name__]
+            name = self.__class__.__name__
+            if name in ["Truncated", "Censored"]:
+                vals = copy(init_vals["Truncated"])
+                vals.update(init_vals[self.dist.__class__.__name__])
+                self._parametrization(**vals)
+            else:
+                args = init_vals[self.__class__.__name__]
+                self._parametrization(**args)
 
-        self.__init__(**args)
-
-        if fixed_lim == "both":
+        if xy_lim == "both":
             xlim = self._finite_endpoints("full")
             xvals = self.xvals("restricted")
             if kind == "pdf":
@@ -398,9 +586,9 @@ class Distribution:
             elif kind == "ppf":
                 max_ppf = self.ppf(0.999)
                 ylim = (-max_ppf * 0.075, max_ppf * 1.5)
-        elif isinstance(fixed_lim, tuple):
-            xlim = fixed_lim[:2]
-            ylim = fixed_lim[2:]
+        elif isinstance(xy_lim, tuple):
+            xlim = xy_lim[:2]
+            ylim = xy_lim[2:]
 
         sliders = {}
         if self.__class__.__name__ == "Categorical":
@@ -422,22 +610,36 @@ class Distribution:
                 args = {list(args.keys())[0].split("_")[0]: values}
                 if np.sum(values) > 1:
                     return None
-            self.__init__(**args)
+
+            self._parametrization(**args)
+
             if kind == "pdf":
                 ax = self.plot_pdf(
-                    legend=False, pointinterval=pointinterval, interval=interval, levels=levels
+                    legend=False,
+                    pointinterval=pointinterval,
+                    interval=interval,
+                    levels=levels,
+                    figsize=figsize,
                 )
             elif kind == "cdf":
                 ax = self.plot_cdf(
-                    legend=False, pointinterval=pointinterval, interval=interval, levels=levels
+                    legend=False,
+                    pointinterval=pointinterval,
+                    interval=interval,
+                    levels=levels,
+                    figsize=figsize,
                 )
             elif kind == "ppf":
                 ax = self.plot_ppf(
-                    legend=False, pointinterval=pointinterval, interval=interval, levels=levels
+                    legend=False,
+                    pointinterval=pointinterval,
+                    interval=interval,
+                    levels=levels,
+                    figsize=figsize,
                 )
-            if fixed_lim != "auto" and kind != "ppf":
+            if xy_lim != "auto" and kind != "ppf":
                 ax.set_xlim(*xlim)
-            if fixed_lim != "auto" and kind != "cdf":
+            if xy_lim != "auto" and kind != "cdf":
                 ax.set_ylim(*ylim)
 
         return interactive(plot, **sliders)
@@ -450,18 +652,36 @@ class Continuous(Distribution):
         super().__init__()
         self.kind = "continuous"
 
-    def xvals(self, support):
+    def xvals(self, support, n_points=1000):
         """Provide x values in the support of the distribution. This is useful for example when
         plotting.
 
         Parameters
         ----------
         support : str
-            Available options are "full" or "restricted".
+            Available options are `full` or `restricted`. If `full` the values will cover the entire
+            support of the distribution, if finite, or the quantiles 0.0001 or 0.9999, if infinite.
+            If `restricted` the values will cover the quantile 0.0001 to 0.9999.
+        n_points : int
+            Number of values to return.
         """
-        lower_ep, upper_ep = self._finite_endpoints(support)
-        x_vals = np.linspace(lower_ep, upper_ep, 1000)
-        return x_vals
+        half_n_points = int(n_points / 2)
+
+        if isinstance(support, tuple):
+            even = np.linspace(*support, n_points)
+            uneven = self.ppf(np.linspace(*self.cdf(support), n_points))
+        else:
+            lower_ep, upper_ep = self.support
+
+            if not np.isfinite(lower_ep) or support == "restricted":
+                lower_ep = 0.0001
+            if not np.isfinite(upper_ep) or support == "restricted":
+                upper_ep = 0.9999
+
+            even = np.linspace(*self.ppf([lower_ep, upper_ep]), half_n_points)
+            uneven = self.ppf(np.linspace(lower_ep, upper_ep, half_n_points))
+
+        return np.sort(np.concatenate([even, uneven]))
 
     def _fit_mle(self, sample, **kwargs):
         """
@@ -484,7 +704,17 @@ class Continuous(Distribution):
         x : array_like
             Values on which to evaluate the pdf
         """
-        return self.rv_frozen.pdf(x, *args, **kwds)
+        return self.pdf(x, *args, **kwds)
+
+    def logpdf(self, x, *args, **kwds):
+        """Probability mass function at x.
+
+        Parameters
+        ----------
+        x : array_like
+            Values on which to evaluate the pdf
+        """
+        return self.logpdf(x, *args, **kwds)
 
 
 class Discrete(Distribution):
@@ -494,13 +724,23 @@ class Discrete(Distribution):
         super().__init__()
         self.kind = "discrete"
 
-    def xvals(self, support):
+    def xvals(self, support, n_points=200):
         """Provide x values in the support of the distribution. This is useful for example when
         plotting.
+
+        Parameters
+        ----------
+        support : str
+            Available options are `full` or `restricted`. If `full` the values will cover the entire
+            support of the distribution, if finite, or the quantiles 0.0001 or 0.9999, if infinite.
+            If `restricted` the values will cover the quantile 0.0001 to 0.9999.
+        n_points : int
+            Number of values to return. The returned values may be fewer than `n_points` if
+            the actual number of discrete values in the support of the distribution is smaller than
+            `n_points`.
         """
         lower_ep, upper_ep = self._finite_endpoints(support)
-        x_vals = np.arange(lower_ep, upper_ep + 1, dtype=int)
-        return x_vals
+        return discrete_xvals(lower_ep, upper_ep, n_points)
 
     def pdf(self, x, *args, **kwds):
         """Probability mass function at x.
@@ -510,4 +750,69 @@ class Discrete(Distribution):
         x : array_like
             Values on which to evaluate the pdf
         """
-        return self.rv_frozen.pmf(x, *args, **kwds)
+        return self.pdf(x, *args, **kwds)
+
+    def logpdf(self, x, *args, **kwds):
+        """Probability mass function at x.
+
+        Parameters
+        ----------
+        x : array_like
+            Values on which to evaluate the pdf
+        """
+        return self.logpdf(x, *args, **kwds)
+
+
+class DistributionTransformer(Distribution):
+    """Base class for distributions that transform other distributions"""
+
+    def __init__(self):
+        super().__init__()
+        self.kind = self.dist.kind
+
+    def xvals(self, support, n_points=None):
+        """Provide x values in the support of the distribution. This is useful for example when
+        plotting.
+
+        Parameters
+        ----------
+        support : str
+            Available options are `full` or `restricted`. If `full` the values will cover the entire
+            support of the distribution, if finite, or the quantiles 0.0001 or 0.9999, if infinite.
+            If `restricted` the values will cover the quantile 0.0001 to 0.9999.
+        n_points : int
+            Number of values to return. For discrete distributions the returned values may be fewer
+            than `n_points` if the actual number of discrete values in the support of the
+            distribution is smaller than `n_points`.
+        """
+        lower_ep, upper_ep = self._finite_endpoints(support)
+
+        if self.kind == "continuous":
+            if n_points is None:
+                n_points = 1000
+            return continuous_xvals(lower_ep, upper_ep, n_points)
+        else:
+            if n_points is None:
+                n_points = 200
+            return discrete_xvals(lower_ep, upper_ep, n_points)
+
+    def skewness(self):
+        """Skewness of the distribution. Not implemented."""
+        return NotImplemented
+
+    def kurtosis(self):
+        """Kurtosis of the distribution. Not implemented."""
+        return NotImplemented
+
+
+def continuous_xvals(lower_ep, upper_ep, n_points):
+    return np.linspace(lower_ep, upper_ep, n_points)
+
+
+def discrete_xvals(lower_ep, upper_ep, n_points):
+    range_x = upper_ep - lower_ep
+    if range_x <= n_points:
+        x_vals = np.arange(lower_ep, upper_ep + 1, dtype=int)
+    else:
+        x_vals = np.linspace(lower_ep, upper_ep + 1, n_points, dtype=int)
+    return x_vals
