@@ -1,18 +1,21 @@
 import inspect
-import logging
 import traceback
 import sys
 
-from IPython import get_ipython
-from ipywidgets import FloatSlider, IntSlider
-from arviz import plot_kde, plot_ecdf, hdi
+try:
+    from IPython import get_ipython
+    from ipywidgets import FloatSlider, IntSlider, FloatText, IntText, Checkbox, ToggleButton
+    from pymc import sample_prior_predictive
+except ImportError:
+    pass
+
+from arviz import plot_kde, plot_ecdf, hdi, extract
+from arviz.stats.density_utils import _kde_linear
 import numpy as np
 import matplotlib.pyplot as plt
 from matplotlib import _pylab_helpers, get_backend
 from matplotlib.ticker import MaxNLocator
-from scipy.interpolate import interp1d, PchipInterpolator
-
-_log = logging.getLogger("preliz")
+from .logging import disable_pymc_sampling_logs
 
 
 def plot_pointinterval(distribution, interval="hdi", levels=None, rotated=False, ax=None):
@@ -66,8 +69,8 @@ def plot_pointinterval(distribution, interval="hdi", levels=None, rotated=False,
             if interval == "eti":
                 func = distribution.eti
 
-            q_tmp = np.concatenate([func(mass=m) for m in levels])
-            median = distribution.rv_frozen.median()
+            q_tmp = np.concatenate([func(mass=m, fmt="none") for m in levels])
+            median = distribution.median()
 
         q_s = []
         if len(levels) == 2:
@@ -99,9 +102,10 @@ def eti(distribution, mass):
     return np.quantile(distribution, (lower, mass + lower))
 
 
-def plot_pdfpmf(dist, moments, pointinterval, interval, levels, support, legend, figsize, ax):
+def plot_pdfpmf(
+    dist, moments, pointinterval, interval, levels, support, legend, color, alpha, figsize, ax
+):
     ax = get_ax(ax, figsize)
-    color = next(ax._get_lines.prop_cycler)["color"]
     if legend is not None:
         label = repr_to_matplotlib(dist)
 
@@ -116,25 +120,43 @@ def plot_pdfpmf(dist, moments, pointinterval, interval, levels, support, legend,
 
     x = dist.xvals(support)
     if dist.kind == "continuous":
+        if dist.__class__.__name__ == "Censored":
+            lower, upper = dist.support
+            disc_vals = [None, None]
+            if np.min(x) <= lower:
+                disc_vals[0] = dist.dist.pdf(lower) * 1.5
+                disc_vals[1] = dist.dist.pdf(upper) * 1.5
+
+            x = x[x != dist.support[0]]
+            x = x[x != dist.support[1]]
+
         density = dist.pdf(x)
         ax.axhline(0, color="0.8", ls="--", zorder=0)
-        ax.plot(x, density, label=label, color=color)
+        p = ax.plot(x, density, label=label, color=color, alpha=alpha)
         ax.set_yticks([])
+
+        if dist.__class__.__name__ == "Censored":
+            if disc_vals[0] is not None:
+                ax.vlines(lower, 0, disc_vals[0], ls="--", color=p[0].get_color(), alpha=alpha)
+            if disc_vals[1] is not None:
+                ax.vlines(upper, 0, disc_vals[1], ls="--", color=p[0].get_color(), alpha=alpha)
+
     else:
         mass = dist.pdf(x)
-        eps = np.clip(dist._finite_endpoints(support), *dist.support)
-        x_c = np.linspace(*eps, 1000)
 
-        if len(x) > 2:
-            interp = PchipInterpolator(x, mass)
+        if dist.__class__.__name__ in ["Categorical", "Bernoulli"]:
+            p = ax.plot(x, mass, "o", label=label, color=color, alpha=alpha)
+            ax.vlines(x, 0, mass, ls="dotted", color=p[0].get_color(), alpha=alpha)
         else:
-            interp = interp1d(x, mass)
+            x_c = np.linspace(x[0], x[-1], 1000)
+            # we compute pmf at non-integer values to get a continuous curve
+            mass_c = np.clip(dist.pdf(x_c), np.min(mass), np.max(mass))
 
-        mass_c = np.clip(interp(x_c), np.min(mass), np.max(mass))
+            p = ax.plot(x_c, mass_c, ls="dotted", color=color, alpha=alpha)
+            ax.plot(x, mass, "o", label=label, color=p[0].get_color(), alpha=alpha)
 
+        ax.xaxis.set_major_locator(MaxNLocator(integer=True))
         ax.axhline(0, color="0.8", ls="--", zorder=0)
-        ax.plot(x_c, mass_c, ls="dotted", color=color)
-        ax.plot(x, mass, "o", label=label, color=color)
 
     if pointinterval:
         plot_pointinterval(dist, interval, levels, ax=ax)
@@ -145,9 +167,10 @@ def plot_pdfpmf(dist, moments, pointinterval, interval, levels, support, legend,
     return ax
 
 
-def plot_cdf(dist, moments, pointinterval, interval, levels, support, legend, figsize, ax):
+def plot_cdf(
+    dist, moments, pointinterval, interval, levels, support, legend, color, alpha, figsize, ax
+):
     ax = get_ax(ax, figsize)
-    color = next(ax._get_lines.prop_cycler)["color"]
     if legend is not None:
         label = repr_to_matplotlib(dist)
 
@@ -170,10 +193,10 @@ def plot_cdf(dist, moments, pointinterval, interval, levels, support, legend, fi
         cdf = dist.cdf(x)
         ax.set_xlim(lower - 0.1, upper + 0.1)
         ax.xaxis.set_major_locator(MaxNLocator(integer=True))
-        ax.step(x, cdf, where="post", label=label, color=color)
+        ax.step(x, cdf, where="post", label=label, color=color, alpha=alpha)
     else:
         cdf = dist.cdf(x)
-        ax.plot(x, cdf, label=label, color=color)
+        ax.plot(x, cdf, label=label, color=color, alpha=alpha)
 
     if pointinterval:
         plot_pointinterval(dist, interval, levels, ax=ax)
@@ -183,10 +206,8 @@ def plot_cdf(dist, moments, pointinterval, interval, levels, support, legend, fi
     return ax
 
 
-def plot_ppf(dist, moments, pointinterval, interval, levels, legend, figsize, ax):
+def plot_ppf(dist, moments, pointinterval, interval, levels, legend, color, alpha, figsize, ax):
     ax = get_ax(ax, figsize)
-    color = next(ax._get_lines.prop_cycler)["color"]
-
     if legend is not None:
         label = repr_to_matplotlib(dist)
 
@@ -200,7 +221,7 @@ def plot_ppf(dist, moments, pointinterval, interval, levels, legend, figsize, ax
         label = None
 
     x = np.linspace(0, 1, 1000)
-    ax.plot(x, dist.ppf(x), label=label, color=color)
+    ax.plot(x, dist.ppf(x), label=label, color=color, alpha=alpha)
     if dist.kind == "discrete":
         ax.yaxis.set_major_locator(MaxNLocator(integer=True))
 
@@ -229,7 +250,7 @@ def side_legend(ax):
 
 
 def repr_to_matplotlib(distribution):
-    string = distribution.__repr__()
+    string = repr(distribution)
     string = string.replace("\x1b[1m", r"$\bf{")
     string = string.replace("\x1b[0m", "}$")
     return string
@@ -245,33 +266,22 @@ def get_moments(dist, moments):
     }
     str_m = []
     seen = []
-    for moment in moments:
+    values = dist.moments(moments)
+    for moment, value in zip(moments, values):
         if moment not in seen:
-            if moment == "d":
-                value = dist.rv_frozen.stats("v") ** 0.5
-            else:
-                value = dist.rv_frozen.stats(moment)
-            if isinstance(value, (np.ndarray, int, float)):
-                str_m.append(f"{names[moment]}={value:.3g}")
+            str_m.append(f"{names[moment]}={value:.3g}")
 
         seen.append(moment)
 
     return "\n" + ", ".join(str_m)
 
 
-def get_slider(name, value, lower, upper, continuous_update=True):
-    if np.isfinite(lower):
-        min_v = lower
-    else:
-        min_v = value - 10
-    if np.isfinite(upper):
-        max_v = upper
-    else:
-        max_v = value + 10
+def get_slider(name, value, lower, upper):
+
+    min_v, max_v, step = generate_range(value, lower, upper)
 
     if isinstance(value, float):
         slider_type = FloatSlider
-        step = (max_v - min_v) / 100
     else:
         slider_type = IntSlider
         step = 1
@@ -283,14 +293,50 @@ def get_slider(name, value, lower, upper, continuous_update=True):
         description=f"{name} ({lower:.0f}, {upper:.0f})",
         value=value,
         style={"description_width": "initial"},
-        continuous_update=continuous_update,
     )
 
     return slider
 
 
-def get_sliders(signature, model):
-    sliders = {}
+def generate_range(value, lower, upper):
+    if value == 0:
+        return -10, 10, 0.1
+
+    power = np.floor(np.log10(abs(value)))
+    order_of_magnitude = 10**power
+    decimal_places = int(-power)
+    std = order_of_magnitude * 10 - order_of_magnitude / 10
+    min_v = round(value - std, decimal_places)
+    max_v = round(value + std, decimal_places)
+
+    min_v = lower if np.isfinite(lower) else min_v
+    max_v = upper if np.isfinite(upper) else max_v
+
+    step = order_of_magnitude / 10
+    return min_v, max_v, step
+
+
+def get_boxes(name, value, lower, upper):
+
+    if isinstance(value, float):
+        text_type = FloatText
+        step = 0.1
+    else:
+        text_type = IntText
+        step = 1
+
+    text = text_type(
+        step=step,
+        description=f"{name} ({lower:.0f}, {upper:.0f})",
+        value=value,
+        style={"description_width": "initial"},
+    )
+
+    return text
+
+
+def get_textboxes(signature, model):
+    textboxes = {}
     for name, param in signature.parameters.items():
         if isinstance(param.default, (int, float)):
             value = float(param.default)
@@ -319,56 +365,163 @@ def get_sliders(signature, model):
         if value is None:
             value = getattr(dist, dist.param_names[idx])
 
-        sliders[name] = get_slider(name, value, lower, upper, continuous_update=False)
-    return sliders
+        textboxes[name] = get_boxes(name, value, lower, upper)
+
+    textboxes["__set_xlim__"] = Checkbox(
+        value=False, description="set xlim", disabled=False, indent=False
+    )
+
+    textboxes["__x_min__"] = FloatText(
+        value=-10,
+        step=0.1,
+        description="x_min",
+        disabled=False,
+    )
+
+    textboxes["__x_max__"] = FloatText(
+        value=10,
+        step=0.1,
+        description="x_max",
+        disabled=False,
+    )
+
+    textboxes["__resample__"] = ToggleButton(
+        value=True,
+        description="Resample",
+        disabled=False,
+        button_style="",
+        tooltip="Resample",
+    )
+
+    return textboxes
 
 
-def plot_decorator(func, iterations, kind_plot):
+def plot_decorator(func, iterations, kind_plot, references, plot_func):
     def looper(*args, **kwargs):
-        results = [func(*args, **kwargs) for _ in range(iterations)]
+        results = []
+        kwargs.pop("__resample__")
+        x_min = kwargs.pop("__x_min__")
+        x_max = kwargs.pop("__x_max__")
+        if not kwargs.pop("__set_xlim__"):
+            x_min = None
+            x_max = None
+            auto = True
+        else:
+            auto = False
+
+        for _ in range(iterations):
+            val = func(*args, **kwargs)
+            if not any(np.isnan(val)):
+                results.append(val)
+        results = np.array(results)
+
         _, ax = plt.subplots()
+        ax.set_xlim(x_min, x_max, auto=auto)
 
-        alpha = max(0.01, 1 - iterations * 0.009)
-
-        if kind_plot == "hist":
-            if results[0].dtype.kind == "i":
-                bins = np.arange(np.min(results), np.max(results) + 1.5) - 0.5
-                if len(bins) < 30:
-                    ax.set_xticks(bins + 0.5)
-            else:
-                bins = "auto"
-            ax.hist(
-                results,
-                alpha=alpha,
-                density=True,
-                color=["C0"] * iterations,
-                bins=bins,
-                histtype="step",
-            )
-            ax.hist(
-                np.concatenate(results),
-                density=True,
-                bins=bins,
-                color="k",
-                ls="--",
-                histtype="step",
-            )
-        elif kind_plot == "kde":
-            for result in results:
-                plot_kde(result, plot_kwargs={"alpha": alpha})
-            plot_kde(np.concatenate(results), plot_kwargs={"color": "k", "ls": "--"})
-        elif kind_plot == "ecdf":
-            plt.plot(
-                np.sort(results, axis=1).T,
-                np.linspace(0, 1, len(results[0]), endpoint=False),
-                color="C0",
-            )
-            a = np.concatenate(results)
-            plt.plot(np.sort(a), np.linspace(0, 1, len(a), endpoint=False), "k--")
+        if plot_func is None:
+            plot_repr(results, kind_plot, references, iterations, ax)
+        else:
+            plot_func(results, ax)
 
     return looper
 
 
+def bambi_plot_decorator(func, iterations, kind_plot, references, plot_func):
+    def looper(*args, **kwargs):
+        kwargs.pop("__resample__")
+        x_min = kwargs.pop("__x_min__")
+        x_max = kwargs.pop("__x_max__")
+        if not kwargs.pop("__set_xlim__"):
+            x_min = None
+            x_max = None
+            auto = True
+        else:
+            auto = False
+
+        model = func(*args, **kwargs)
+        model.build()
+        with disable_pymc_sampling_logs():
+            idata = model.prior_predictive(iterations)
+        results = extract(idata, group="prior_predictive")[model.response_name].values.T
+
+        _, ax = plt.subplots()
+        ax.set_xlim(x_min, x_max, auto=auto)
+        if plot_func is None:
+            plot_repr(results, kind_plot, references, iterations, ax)
+        else:
+            plot_func(results, ax)
+
+    return looper
+
+
+def pymc_plot_decorator(func, iterations, kind_plot, references, plot_func):
+    def looper(*args, **kwargs):
+        kwargs.pop("__resample__")
+        x_min = kwargs.pop("__x_min__")
+        x_max = kwargs.pop("__x_max__")
+        if not kwargs.pop("__set_xlim__"):
+            x_min = None
+            x_max = None
+            auto = True
+        else:
+            auto = False
+        with func(*args, **kwargs) as model:
+            obs_name = model.observed_RVs[0].name
+            with disable_pymc_sampling_logs():
+                idata = sample_prior_predictive(samples=iterations)
+            results = extract(idata, group="prior_predictive")[obs_name].values.T
+        _, ax = plt.subplots()
+        ax.set_xlim(x_min, x_max, auto=auto)
+        if plot_func is None:
+            plot_repr(results, kind_plot, references, iterations, ax)
+        else:
+            plot_func(results, ax)
+
+    return looper
+
+
+def plot_repr(results, kind_plot, references, iterations, ax):
+    alpha = max(0.01, 1 - iterations * 0.009)
+
+    if kind_plot == "hist":
+        if results[0].dtype.kind == "i":
+            bins = np.arange(np.min(results), np.max(results) + 1.5) - 0.5
+            if len(bins) < 30:
+                ax.set_xticks(bins + 0.5)
+        else:
+            bins = "auto"
+        ax.hist(
+            results.T,
+            alpha=alpha,
+            density=True,
+            color=["0.5"] * iterations,
+            bins=bins,
+            histtype="step",
+        )
+        ax.hist(
+            np.concatenate(results),
+            density=True,
+            bins=bins,
+            color="k",
+            ls="--",
+            histtype="step",
+        )
+    elif kind_plot == "kde":
+        for result in results:
+            ax.plot(*_kde_linear(result, grid_len=100), "0.5", alpha=alpha)
+        ax.plot(*_kde_linear(np.concatenate(results), grid_len=100), "k--")
+    elif kind_plot == "ecdf":
+        ax.plot(
+            np.sort(results, axis=1).T,
+            np.linspace(0, 1, len(results[0]), endpoint=False),
+            color="0.5",
+        )
+        a = np.concatenate(results)
+        ax.plot(np.sort(a), np.linspace(0, 1, len(a), endpoint=False), "k--")
+
+    plot_references(references, ax)
+
+
 def plot_pp_samples(pp_samples, pp_samples_idxs, references, kind="pdf", sharex=True, fig=None):
     row_colum = int(np.ceil(len(pp_samples_idxs) ** 0.5))
 
@@ -388,8 +541,7 @@ def plot_pp_samples(pp_samples, pp_samples_idxs, references, kind="pdf", sharex=
 
     for ax, idx in zip(axes, pp_samples_idxs):
         ax.clear()
-        for ref in references:
-            ax.axvline(ref, ls="--", color="0.5")
+        plot_references(references, ax)
         ax.relim()
 
         sample = pp_samples[idx]
@@ -440,8 +592,7 @@ def plot_pp_mean(pp_samples, selected, references=None, kind="pdf", fig_pp_mean=
     else:
         sample = pp_samples.ravel()
 
-    for ref in references:
-        ax_pp_mean.axvline(ref, ls="--", color="0.5")
+    plot_references(references, ax_pp_mean)
 
     if kind == "pdf":
         plot_kde(
@@ -465,6 +616,20 @@ def plot_pp_mean(pp_samples, selected, references=None, kind="pdf", fig_pp_mean=
     return fig_pp_mean
 
 
+def plot_references(references, ax):
+    if references is not None:
+        if isinstance(references, dict):
+            max_value = ax.get_ylim()[1]
+            for label, ref in references.items():
+                ax.text(ref, max_value * 0.2, label, rotation=90, bbox={"color": "w", "alpha": 0.5})
+                ax.axvline(ref, ls="--", color="0.5")
+        else:
+            if isinstance(references, (float, int)):
+                references = [references]
+            for ref in references:
+                ax.axvline(ref, ls="--", color="0.5")
+
+
 def check_inside_notebook(need_widget=False):
     shell = get_ipython()
     name = inspect.currentframe().f_back.f_code.co_name
@@ -482,3 +647,44 @@ def check_inside_notebook(need_widget=False):
         tb_as_str = traceback.format_exc()
         # Print only the last line of the traceback, which contains the error message
         print(tb_as_str.strip().rsplit("\n", maxsplit=1)[-1], file=sys.stdout)
+
+
+def representations(fitted_dist, kind_plot, ax):
+    if kind_plot == "pdf":
+        fitted_dist.plot_pdf(pointinterval=True, legend="title", ax=ax)
+        ax.set_yticks([])
+
+        for bound in fitted_dist.support:
+            if np.isfinite(bound):
+                ax.plot(bound, 0, "ko")
+
+    elif kind_plot == "cdf":
+        fitted_dist.plot_cdf(pointinterval=True, legend="title", ax=ax)
+
+    elif kind_plot == "ppf":
+        fitted_dist.plot_ppf(pointinterval=True, legend="title", ax=ax)
+        ax.set_xlim(-0.01, 1)
+
+
+def create_figure(figsize):
+    """
+    Initialize a matplotlib figure with one subplot
+    """
+    fig, axes = plt.subplots(1, 1, figsize=figsize, constrained_layout=True)
+    axes.set_yticks([])
+    fig.canvas.header_visible = False
+    fig.canvas.footer_visible = False
+    fig.canvas.toolbar_position = "right"
+
+    return fig, axes
+
+
+def reset_dist_panel(ax, yticks):
+    """
+    Clean the distribution subplot
+    """
+    ax.cla()
+    if yticks:
+        ax.set_yticks([])
+    ax.relim()
+    ax.autoscale_view()
