@@ -1,6 +1,7 @@
 # pylint: disable=redefined-outer-name
 
 import pytest
+from pymc import Model
 from numpy.testing import assert_almost_equal
 import numpy as np
 from test_helper import run_notebook
@@ -8,6 +9,7 @@ from test_helper import run_notebook
 from preliz.distributions import (
     AsymmetricLaplace,
     Beta,
+    BetaScaled,
     Cauchy,
     ChiSquared,
     Gamma,
@@ -16,10 +18,12 @@ from preliz.distributions import (
     Exponential,
     HalfCauchy,
     HalfNormal,
-    HalfStudent,
+    HalfStudentT,
     InverseGamma,
+    Kumaraswamy,
     Laplace,
     Logistic,
+    LogLogistic,
     LogNormal,
     LogitNormal,
     Moyal,
@@ -27,7 +31,7 @@ from preliz.distributions import (
     Pareto,
     Rice,
     SkewNormal,
-    Student,
+    StudentT,
     Triangular,
     TruncatedNormal,
     Uniform,
@@ -38,10 +42,14 @@ from preliz.distributions import (
     BetaBinomial,
     Binomial,
     DiscreteUniform,
+    DiscreteWeibull,
     Geometric,
     NegativeBinomial,
     Poisson,
+    ZeroInflatedNegativeBinomial,
     ZeroInflatedPoisson,
+    Dirichlet,
+    MvNormal,
 )
 
 
@@ -50,6 +58,11 @@ def a_few_poissons():
     return Poisson(), Poisson([1.0, 2.0]), Poisson(4.5)
 
 
+@pytest.fixture(scope="session")
+def multivariates():
+    return Dirichlet(), Dirichlet([1.0, 2.0, 1.0]), MvNormal(np.zeros(2), np.eye(2))
+
+
 @pytest.mark.parametrize(
     "distribution, params",
     [
@@ -61,10 +74,12 @@ def a_few_poissons():
         (Gamma, (1, 0.5)),
         (Gumbel, (1, 2)),
         (HalfNormal, (1,)),
-        (HalfStudent, (100, 1)),
+        (HalfStudentT, (100, 1)),
         (InverseGamma, (3, 1)),
+        (Kumaraswamy, (5, 2)),
         (Laplace, (0, 1)),
         (Logistic, (1, 2)),
+        (LogLogistic, (1, 5)),
         (LogNormal, (0, 0.5)),
         (LogitNormal, (0, 0.5)),
         (Moyal, (1, 2)),
@@ -72,7 +87,7 @@ def a_few_poissons():
         (Pareto, (5, 1)),
         (Rice, (4, 1)),
         (SkewNormal, (0, 1, 0)),
-        (Student, (100, 0, 1)),
+        (StudentT, (100, 0, 1)),
         (Triangular, (-2, 3, 7)),
         (TruncatedNormal, (0, 1, -np.inf, np.inf)),
         (Uniform, (0, 1)),
@@ -84,6 +99,7 @@ def a_few_poissons():
         (Binomial, (2, 0.5)),
         (Binomial, (2, 0.1)),
         (DiscreteUniform, (0, 1)),
+        (DiscreteWeibull, (0.5, 3)),
         (Geometric, (0.75,)),
         (NegativeBinomial, (8, 4)),
         (Poisson, (4.5,)),
@@ -100,13 +116,23 @@ def test_moments(distribution, params):
     dist = distribution(*params)
     dist_ = distribution()
 
-    dist_._fit_moments(dist.rv_frozen.mean(), dist.rv_frozen.std())
+    dist_._fit_moments(dist.mean(), dist.std())
 
     tol = 5
-    if dist.__class__.__name__ in ["BetaBinomial", "Binomial", "LogitNormal", "Rice", "Student"]:
+    if dist.__class__.__name__ in [
+        "BetaBinomial",
+        "Binomial",
+        "DiscreteWeibull",
+        "ExGaussian",
+        "NegativeBinomial",
+        "Kumaraswamy",
+        "LogitNormal",
+        "Rice",
+        "StudentT",
+    ]:
         tol = 0
-    assert_almost_equal(dist.rv_frozen.mean(), dist_.rv_frozen.mean(), tol)
-    assert_almost_equal(dist.rv_frozen.std(), dist_.rv_frozen.std(), tol)
+    assert_almost_equal(dist.mean(), dist_.mean(), tol)
+    assert_almost_equal(dist.std(), dist_.std(), tol)
     assert_almost_equal(params, dist_.params, 0)
 
 
@@ -115,6 +141,7 @@ def test_moments(distribution, params):
     [
         (AsymmetricLaplace, (1, 4, 3)),
         (Beta, (2, 5)),
+        (BetaScaled, (2, 2, -1, 2)),
         (Cauchy, (0, 1)),
         (ChiSquared, (1,)),
         (ExGaussian, (0, 1, 3)),
@@ -123,8 +150,9 @@ def test_moments(distribution, params):
         (Gumbel, (0, 1)),
         (HalfCauchy, (1,)),
         (HalfNormal, (1,)),
-        (HalfStudent, (100, 1)),
+        (HalfStudentT, (100, 1)),
         (InverseGamma, (3, 0.5)),
+        (Kumaraswamy, (2, 3)),
         (Laplace, (0, 1)),
         (Logistic, (0, 1)),
         (LogNormal, (0, 0.5)),
@@ -135,8 +163,8 @@ def test_moments(distribution, params):
         (Rice, (1, 2)),
         (SkewNormal, (0, 1, 0)),
         (SkewNormal, (0, 1, -1)),
-        (Student, (4, 0, 1)),
-        (Student, (1000, 0, 1)),
+        (StudentT, (4, 0, 1)),
+        (StudentT, (1000, 0, 1)),
         (Triangular, (-3, 0, 5)),
         (TruncatedNormal, (0, 1, -1, 1)),
         (Uniform, (0, 1)),
@@ -148,9 +176,11 @@ def test_moments(distribution, params):
         (Binomial, (2, 0.5)),
         (Binomial, (2, 0.1)),
         (DiscreteUniform, (0, 1)),
+        (DiscreteWeibull, (0.1, 1.5)),
         (Geometric, (0.5,)),
         (NegativeBinomial, (8, 4)),
         (Poisson, (4.5,)),
+        (ZeroInflatedNegativeBinomial, (0.7, 8, 4)),
         (
             ZeroInflatedPoisson,
             (
@@ -161,32 +191,40 @@ def test_moments(distribution, params):
     ],
 )
 def test_mle(distribution, params):
-    dist = distribution(*params)
-    sample = dist.rv_frozen.rvs(20000)
-    dist_ = distribution()
-    dist_._fit_mle(sample)
+    for _ in range(10):
+        dist = distribution(*params)
+        sample = dist.rvs(20_000)
+        dist_ = distribution()
+        dist_._fit_mle(sample)
 
-    if dist.__class__.__name__ == "Pareto":
-        tol = 0
+        if dist.__class__.__name__ in ["Pareto", "ExGaussian"]:
+            tol = 0
+        else:
+            tol = 1
+        try:
+            assert_almost_equal(dist.mean(), dist_.mean(), tol)
+            assert_almost_equal(dist.std(), dist_.std(), tol)
+            if dist.__class__.__name__ == "StudentT":
+                assert_almost_equal(params[1:], dist_.params[1:], 0)
+            else:
+                assert_almost_equal(params, dist_.params, 0)
+            break
+        except AssertionError:
+            pass
     else:
-        tol = 1
-    assert_almost_equal(dist.rv_frozen.mean(), dist_.rv_frozen.mean(), tol)
-    assert_almost_equal(dist.rv_frozen.std(), dist_.rv_frozen.std(), tol)
-    if dist.__class__.__name__ == "Student":
-        assert_almost_equal(params[1:], dist_.params[1:], 0)
-    else:
-        assert_almost_equal(params, dist_.params, 0)
+        raise AssertionError("Test failed after 10 attempts")
 
 
 @pytest.mark.parametrize("fmt", (".2f", ".1g"))
+@pytest.mark.parametrize("interval", ("hdi", "eti", [0.25, 0.75]))
 @pytest.mark.parametrize("mass", (0.5, 0.95))
-def test_summary_args(fmt, mass):
-    result = Normal(0, 1).summary(fmt, mass)
+def test_summary_args(mass, interval, fmt):
+    result = Normal(0, 1).summary(mass=mass, interval=interval, fmt=fmt)
     assert result.mean == 0
     assert result.std == 1
 
 
-def test_summary_valid(a_few_poissons):
+def test_summary_univariate_valid(a_few_poissons):
     d_0, d_1, d_2 = a_few_poissons
     with pytest.raises(ValueError):
         d_0.summary()
@@ -197,7 +235,21 @@ def test_summary_valid(a_few_poissons):
     assert result.mean == 4.5
     assert result.std == 2.12
     assert result.lower == 1.0
-    assert result.upper == 9.0
+    assert result.upper == 8.0
+
+
+def test_summary_multivariate_valid(multivariates):
+    d_0, d_1, m_0 = multivariates
+    with pytest.raises(ValueError):
+        d_0.summary()
+    result = d_1.summary()
+    assert result.__class__.__name__ == "Dirichlet"
+    assert_almost_equal(result.mean, [0.25, 0.5, 0.25])
+    assert_almost_equal(result.std, [0.193, 0.223, 0.193], decimal=3)
+    result = m_0.summary()
+    assert result.__class__.__name__ == "MvNormal"
+    assert_almost_equal(result.mean, [0, 0])
+    assert_almost_equal(result.std, [1, 1])
 
 
 def test_eti(a_few_poissons):
@@ -220,14 +272,6 @@ def test_hdi(a_few_poissons):
     assert result == (1.0, 8.0)
 
 
-def test_rvs(a_few_poissons):
-    _, d_1, d_2 = a_few_poissons
-    result0 = d_2.rvs()
-    result1 = d_1.rvs()
-    assert isinstance(result0, int)
-    assert len(result1) == 2
-
-
 def test_cdf(a_few_poissons):
     _, d_1, d_2 = a_few_poissons
     result1 = d_1.cdf(1)
@@ -250,3 +294,12 @@ def test_plot_interactive(capsys, a_few_poissons):
     captured = capsys.readouterr()
     assert "RuntimeError" in captured.out
     run_notebook("plot_interactive.ipynb")
+
+
+def test_to_pymc():
+    with Model() as model:
+        Gamma(1, 1).to_pymc("a", shape=(2, 2))
+
+    assert model.basic_RVs[0].name == "a"
+    assert model.basic_RVs[0].ndim == 2
+    assert Normal(0, 1).to_pymc(shape=2).ndim == 1
